@@ -246,7 +246,7 @@ static char *PNMString(Image *image,char *string,const size_t extent)
           return((char *) NULL);
         break;
       }
-    string[i]=c;
+    string[i]=(char) c;
     if (c == '\n' || c == '\r')
       break;
   }
@@ -409,7 +409,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           do
           {
             if ((size_t) (p-keyword) < (MagickPathExtent-1))
-              *p++=c;
+              *p++=(char) c;
             c=ReadBlobByte(image);
           } while (isalnum((int) ((unsigned char) c)));
           *p='\0';
@@ -421,7 +421,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
           while (isalnum((int) ((unsigned char) c)) || (c == '_'))
           {
             if ((size_t) (p-value) < (MagickPathExtent-1))
-              *p++=c;
+              *p++=(char) c;
             c=ReadBlobByte(image);
           }
           *p='\0';
@@ -535,7 +535,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
               0 ? QuantumRange : 0,q);
             if (EOFBlob(image) != MagickFalse)
               break;
-            q+=GetPixelChannels(image);
+            q+=(ptrdiff_t) GetPixelChannels(image);
           }
           if (SyncAuthenticPixels(image,exception) == MagickFalse)
             break;
@@ -579,7 +579,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
             if (EOFBlob(image) != MagickFalse)
               break;
             SetPixelGray(image,intensity,q);
-            q+=GetPixelChannels(image);
+            q+=(ptrdiff_t) GetPixelChannels(image);
           }
           if (SyncAuthenticPixels(image,exception) == MagickFalse)
             break;
@@ -634,7 +634,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
             if ((is_mono != MagickFalse) &&
                 (IsPixelMonochrome(image,q) == MagickFalse))
               is_mono=MagickFalse;
-            q+=GetPixelChannels(image);
+            q+=(ptrdiff_t) GetPixelChannels(image);
           }
           if (SyncAuthenticPixels(image,exception) == MagickFalse)
             break;
@@ -789,7 +789,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   {
                     p=PushCharPixel(p,&pixel);
                     SetPixelGray(image,ScaleAnyToQuantum(pixel,max_value),q);
-                    q+=GetPixelChannels(image);
+                    q+=(ptrdiff_t) GetPixelChannels(image);
                   }
                   break;
                 }
@@ -808,7 +808,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   {
                     p=PushShortPixel(MSBEndian,p,&pixel);
                     SetPixelGray(image,ScaleAnyToQuantum(pixel,max_value),q);
-                    q+=GetPixelChannels(image);
+                    q+=(ptrdiff_t) GetPixelChannels(image);
                   }
                   break;
                 }
@@ -821,7 +821,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   {
                     p=PushLongPixel(MSBEndian,p,&pixel);
                     SetPixelGray(image,ScaleAnyToQuantum(pixel,max_value),q);
-                    q+=GetPixelChannels(image);
+                    q+=(ptrdiff_t) GetPixelChannels(image);
                   }
                   break;
                 }
@@ -912,7 +912,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 if ((is_mono != MagickFalse) &&
                     (IsPixelMonochrome(image,q) == MagickFalse))
                   is_mono=MagickFalse;
-                q+=GetPixelChannels(image);
+                q+=(ptrdiff_t) GetPixelChannels(image);
               }
               break;
             }
@@ -930,7 +930,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 if ((is_mono != MagickFalse) &&
                     (IsPixelMonochrome(image,q) == MagickFalse))
                   is_mono=MagickFalse;
-                q+=GetPixelChannels(image);
+                q+=(ptrdiff_t) GetPixelChannels(image);
               }
               break;
             }
@@ -960,7 +960,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 if ((is_mono != MagickFalse) &&
                     (IsPixelMonochrome(image,q) == MagickFalse))
                   is_mono=MagickFalse;
-                q+=GetPixelChannels(image);
+                q+=(ptrdiff_t) GetPixelChannels(image);
               }
               break;
             }
@@ -984,7 +984,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 if ((is_mono != MagickFalse) &&
                     (IsPixelMonochrome(image,q) == MagickFalse))
                   is_mono=MagickFalse;
-                q+=GetPixelChannels(image);
+                q+=(ptrdiff_t) GetPixelChannels(image);
               }
               break;
             }
@@ -1008,7 +1008,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 if ((is_mono != MagickFalse) &&
                     (IsPixelMonochrome(image,q) == MagickFalse))
                   is_mono=MagickFalse;
-                q+=GetPixelChannels(image);
+                q+=(ptrdiff_t) GetPixelChannels(image);
               }
               break;
             }
@@ -1032,7 +1032,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 if ((is_mono != MagickFalse) &&
                     (IsPixelMonochrome(image,q) == MagickFalse))
                   is_mono=MagickFalse;
-                q+=GetPixelChannels(image);
+                q+=(ptrdiff_t) GetPixelChannels(image);
               }
               break;
             }
@@ -1165,7 +1165,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                               SetPixelAlpha(image,QuantumRange-
                                 ScaleAnyToQuantum(pixel,max_value),q);
                           }
-                        q+=GetPixelChannels(image);
+                        q+=(ptrdiff_t) GetPixelChannels(image);
                       }
                       break;
                     }
@@ -1193,7 +1193,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                             SetPixelAlpha(image,ScaleAnyToQuantum(pixel,
                               max_value),q);
                           }
-                        q+=GetPixelChannels(image);
+                        q+=(ptrdiff_t) GetPixelChannels(image);
                       }
                       break;
                     }
@@ -1214,7 +1214,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                             SetPixelAlpha(image,ScaleAnyToQuantum(pixel,
                               max_value),q);
                           }
-                        q+=GetPixelChannels(image);
+                        q+=(ptrdiff_t) GetPixelChannels(image);
                       }
                     }
                   }
@@ -1257,7 +1257,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                             SetPixelAlpha(image,ScaleAnyToQuantum(pixel,
                               max_value),q);
                           }
-                        q+=GetPixelChannels(image);
+                        q+=(ptrdiff_t) GetPixelChannels(image);
                       }
                       break;
                     }
@@ -1293,7 +1293,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                             SetPixelAlpha(image,ScaleAnyToQuantum(pixel,
                               max_value),q);
                           }
-                        q+=GetPixelChannels(image);
+                        q+=(ptrdiff_t) GetPixelChannels(image);
                       }
                       break;
                     }
@@ -1323,7 +1323,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                             SetPixelAlpha(image,ScaleAnyToQuantum(pixel,
                               max_value),q);
                           }
-                        q+=GetPixelChannels(image);
+                        q+=(ptrdiff_t) GetPixelChannels(image);
                       }
                       break;
                     }
@@ -1363,7 +1363,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                             SetPixelAlpha(image,ScaleAnyToQuantum(pixel,
                               max_value),q);
                           }
-                        q+=GetPixelChannels(image);
+                        q+=(ptrdiff_t) GetPixelChannels(image);
                       }
                       break;
                     }
@@ -1396,7 +1396,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                             SetPixelAlpha(image,ScaleAnyToQuantum(pixel,
                               max_value),q);
                           }
-                        q+=GetPixelChannels(image);
+                        q+=(ptrdiff_t) GetPixelChannels(image);
                       }
                       break;
                     }
@@ -1423,7 +1423,7 @@ static Image *ReadPNMImage(const ImageInfo *image_info,ExceptionInfo *exception)
                             SetPixelAlpha(image,ScaleAnyToQuantum(pixel,
                               max_value),q);
                           }
-                        q+=GetPixelChannels(image);
+                        q+=(ptrdiff_t) GetPixelChannels(image);
                       }
                       break;
                     }
@@ -2007,12 +2007,13 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
       case '1':
       {
         unsigned char
-          pixels[2048];
+          pixels[70];
 
         /*
           Convert image to a PBM image.
         */
         (void) SetImageType(image,BilevelType,exception);
+        extent=1;
         q=pixels;
         for (y=0; y < (ssize_t) image->rows; y++)
         {
@@ -2027,20 +2028,21 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
             break;
           for (x=0; x < (ssize_t) image->columns; x++)
           {
+            if (q != pixels)
+              {
+                if ((size_t) (q-pixels+(ssize_t) extent+1) < sizeof(pixels))
+                  *q++=' ';
+                else
+                  {
+                    *q++='\n';
+                    (void) WriteBlob(image,(size_t) (q-pixels),pixels);
+                    q=pixels;
+                  }
+              }
             *q++=(unsigned char) (GetPixelLuma(image,p) >= ((double)
               QuantumRange/2.0) ? '0' : '1');
-            if ((q-pixels+2) >= (ssize_t) sizeof(pixels))
-              {
-                *q++='\n';
-                (void) WriteBlob(image,(size_t) (q-pixels),pixels);
-                q=pixels;
-              }
-            *q++=' ';
-            p+=GetPixelChannels(image);
+            p+=(ptrdiff_t) GetPixelChannels(image);
           }
-          *q++='\n';
-          (void) WriteBlob(image,(size_t) (q-pixels),pixels);
-          q=pixels;
           if (image->previous == (Image *) NULL)
             {
               status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
@@ -2059,7 +2061,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
       case '2':
       {
         unsigned char
-          pixels[2048];
+          pixels[70];
 
         /*
           Convert image to a PGM image.
@@ -2087,29 +2089,31 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
           {
             index=ClampToQuantum(GetPixelLuma(image,p));
             if (image->depth <= 8)
-              count=(ssize_t) FormatLocaleString(buffer,MagickPathExtent,"%u ",
+              count=(ssize_t) FormatLocaleString(buffer,MagickPathExtent,"%u",
                 ScaleQuantumToChar(index));
             else
               if (image->depth <= 16)
                 count=(ssize_t) FormatLocaleString(buffer,MagickPathExtent,
-                  "%u ",ScaleQuantumToShort(index));
+                  "%u",ScaleQuantumToShort(index));
               else
                 count=(ssize_t) FormatLocaleString(buffer,MagickPathExtent,
-                  "%u ",ScaleQuantumToLong(index));
+                  "%u",ScaleQuantumToLong(index));
             extent=(size_t) count;
-            if ((size_t) (q-pixels+(ssize_t) extent+1) >= sizeof(pixels))
+            if (q != pixels)
               {
-                *q++='\n';
-                (void) WriteBlob(image,(size_t) (q-pixels),pixels);
-                q=pixels;
+                if ((size_t) (q-pixels+(ssize_t) extent+1) < sizeof(pixels))
+                  *q++=' ';
+                else
+                  {
+                    *q++='\n';
+                    (void) WriteBlob(image,(size_t) (q-pixels),pixels);
+                    q=pixels;
+                  }
               }
             (void) memcpy((char *) q,buffer,extent);
-            q+=extent;
-            p+=GetPixelChannels(image);
+            q+=(ptrdiff_t) extent;
+            p+=(ptrdiff_t) GetPixelChannels(image);
           }
-          *q++='\n';
-          (void) WriteBlob(image,(size_t) (q-pixels),pixels);
-          q=pixels;
           if (image->previous == (Image *) NULL)
             {
               status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
@@ -2128,7 +2132,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
       case '3':
       {
         unsigned char
-          pixels[2048];
+          pixels[70];
 
         /*
           Convert image to a PNM image.
@@ -2158,34 +2162,36 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
           {
             if (image->depth <= 8)
               count=(ssize_t) FormatLocaleString(buffer,MagickPathExtent,
-                "%u %u %u ",ScaleQuantumToChar(GetPixelRed(image,p)),
+                "%u %u %u",ScaleQuantumToChar(GetPixelRed(image,p)),
                 ScaleQuantumToChar(GetPixelGreen(image,p)),
                 ScaleQuantumToChar(GetPixelBlue(image,p)));
             else
               if (image->depth <= 16)
                 count=(ssize_t) FormatLocaleString(buffer,MagickPathExtent,
-                  "%u %u %u ",ScaleQuantumToShort(GetPixelRed(image,p)),
+                  "%u %u %u",ScaleQuantumToShort(GetPixelRed(image,p)),
                   ScaleQuantumToShort(GetPixelGreen(image,p)),
                   ScaleQuantumToShort(GetPixelBlue(image,p)));
               else
                 count=(ssize_t) FormatLocaleString(buffer,MagickPathExtent,
-                  "%u %u %u ",ScaleQuantumToLong(GetPixelRed(image,p)),
+                  "%u %u %u",ScaleQuantumToLong(GetPixelRed(image,p)),
                   ScaleQuantumToLong(GetPixelGreen(image,p)),
                   ScaleQuantumToLong(GetPixelBlue(image,p)));
             extent=(size_t) count;
-            if ((size_t) (q-pixels+(ssize_t) extent+2) >= sizeof(pixels))
+            if (q != pixels)
               {
-                *q++='\n';
-                (void) WriteBlob(image,(size_t) (q-pixels),pixels);
-                q=pixels;
+                if ((size_t) (q-pixels+(ssize_t) extent+1) < sizeof(pixels))
+                  *q++=' ';
+                else
+                  {
+                    *q++='\n';
+                    (void) WriteBlob(image,(size_t) (q-pixels),pixels);
+                    q=pixels;
+                  }
               }
             (void) memcpy((char *) q,buffer,extent);
-            q+=extent;
-            p+=GetPixelChannels(image);
+            q+=(ptrdiff_t) extent;
+            p+=(ptrdiff_t) GetPixelChannels(image);
           }
-          *q++='\n';
-          (void) WriteBlob(image,(size_t) (q-pixels),pixels);
-          q=pixels;
           if (image->previous == (Image *) NULL)
             {
               status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
@@ -2300,7 +2306,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
                             max_value);
                       }
                     q=PopCharPixel((unsigned char) pixel,q);
-                    p+=GetPixelChannels(image);
+                    p+=(ptrdiff_t) GetPixelChannels(image);
                   }
                   extent=(size_t) (q-pixels);
                   break;
@@ -2321,7 +2327,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
                             max_value);
                       }
                     q=PopShortPixel(MSBEndian,(unsigned short) pixel,q);
-                    p+=GetPixelChannels(image);
+                    p+=(ptrdiff_t) GetPixelChannels(image);
                   }
                   extent=(size_t) (q-pixels);
                   break;
@@ -2339,7 +2345,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
                       pixel=ScaleQuantumToAny(GetPixelRed(image,p),max_value);
                   }
                 q=PopLongPixel(MSBEndian,(unsigned int) pixel,q);
-                p+=GetPixelChannels(image);
+                p+=(ptrdiff_t) GetPixelChannels(image);
               }
               extent=(size_t) (q-pixels);
               break;
@@ -2414,7 +2420,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
                     q=PopCharPixel((unsigned char) pixel,q);
                     pixel=ScaleQuantumToAny(GetPixelBlue(image,p),max_value);
                     q=PopCharPixel((unsigned char) pixel,q);
-                    p+=GetPixelChannels(image);
+                    p+=(ptrdiff_t) GetPixelChannels(image);
                   }
                   extent=(size_t) (q-pixels);
                   break;
@@ -2429,7 +2435,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
                     q=PopShortPixel(MSBEndian,(unsigned short) pixel,q);
                     pixel=ScaleQuantumToAny(GetPixelBlue(image,p),max_value);
                     q=PopShortPixel(MSBEndian,(unsigned short) pixel,q);
-                    p+=GetPixelChannels(image);
+                    p+=(ptrdiff_t) GetPixelChannels(image);
                   }
                   extent=(size_t) (q-pixels);
                   break;
@@ -2442,7 +2448,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
                 q=PopLongPixel(MSBEndian,(unsigned int) pixel,q);
                 pixel=ScaleQuantumToAny(GetPixelBlue(image,p),max_value);
                 q=PopLongPixel(MSBEndian,(unsigned int) pixel,q);
-                p+=GetPixelChannels(image);
+                p+=(ptrdiff_t) GetPixelChannels(image);
               }
               extent=(size_t) (q-pixels);
               break;
@@ -2519,7 +2525,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
                               GetPixelAlpha(image,p),max_value);
                             q=PopCharPixel((unsigned char) pixel,q);
                           }
-                        p+=GetPixelChannels(image);
+                        p+=(ptrdiff_t) GetPixelChannels(image);
                       }
                       break;
                     }
@@ -2536,7 +2542,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
                               GetPixelAlpha(image,p),max_value);
                             q=PopShortPixel(MSBEndian,(unsigned short) pixel,q);
                           }
-                        p+=GetPixelChannels(image);
+                        p+=(ptrdiff_t) GetPixelChannels(image);
                       }
                       break;
                     }
@@ -2551,7 +2557,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
                           GetPixelAlpha(image,p),max_value);
                         q=PopLongPixel(MSBEndian,(unsigned int) pixel,q);
                       }
-                    p+=GetPixelChannels(image);
+                    p+=(ptrdiff_t) GetPixelChannels(image);
                   }
                   break;
                 }
@@ -2579,7 +2585,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
                               max_value);
                             q=PopCharPixel((unsigned char) pixel,q);
                           }
-                        p+=GetPixelChannels(image);
+                        p+=(ptrdiff_t) GetPixelChannels(image);
                       }
                       break;
                     }
@@ -2604,7 +2610,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
                               max_value);
                             q=PopShortPixel(MSBEndian,(unsigned short) pixel,q);
                           }
-                        p+=GetPixelChannels(image);
+                        p+=(ptrdiff_t) GetPixelChannels(image);
                       }
                       break;
                     }
@@ -2624,7 +2630,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
                           max_value);
                         q=PopLongPixel(MSBEndian,(unsigned int) pixel,q);
                       }
-                    p+=GetPixelChannels(image);
+                    p+=(ptrdiff_t) GetPixelChannels(image);
                   }
                   break;
                 }
@@ -2648,7 +2654,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
                               max_value);
                             q=PopCharPixel((unsigned char) pixel,q);
                           }
-                        p+=GetPixelChannels(image);
+                        p+=(ptrdiff_t) GetPixelChannels(image);
                       }
                       break;
                     }
@@ -2670,7 +2676,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
                               max_value);
                             q=PopShortPixel(MSBEndian,(unsigned short) pixel,q);
                           }
-                        p+=GetPixelChannels(image);
+                        p+=(ptrdiff_t) GetPixelChannels(image);
                       }
                       break;
                     }
@@ -2688,7 +2694,7 @@ static MagickBooleanType WritePNMImage(const ImageInfo *image_info,Image *image,
                           max_value);
                         q=PopLongPixel(MSBEndian,(unsigned int) pixel,q);
                       }
-                    p+=GetPixelChannels(image);
+                    p+=(ptrdiff_t) GetPixelChannels(image);
                   }
                   break;
                 }
